#!/usr/bin/perl -w
# Written on May 3, 1024 by Matyas (Matthew) Cserhati

my $usage = << "EOF";

Usage: $0
	-g genome sequence
        -o output file
	-k length of motifs to find

EOF

use Getopt::Long;

my %par = (
                  );

&GetOptions("g=s" => \$par{genome},
	    "k=i" => \$par{k},
	    "o=s" => \$par{outfile});

($par{genome} && $par{outfile} && $par{k}) or die $usage;

&go (\%par );

print STDERR "$0 done\n";

###############################################

sub go {
    	my ( $par ) = @_;
	&readgenome($par->{genome}, $par->{outfile}, $par->{k});
}

sub readgenome {
	my ($genome, $ofile, $k) = @_;
	my %bp = (); $bp{'A'} = 0; $bp{'C'} = 0; $bp{'G'} = 0; $bp{'T'} = 0;
	my %motifs = ();
	my $gensize = 0; my $c = 0;
	my $motif = "";
	my $nseq = 0;

	open (G, "<$genome") or die "Cannot open infile $genome $!\n";
	while (<G>) {
		chomp;
		$_ = uc($_);
		if (/^>/) {
			if ($nseq > 0) {
                        	if (!(defined($motifs{$motif}))) {
 	                       		$motifs{$motif} = 1;
                        	} else {
                               		$motifs{$motif}++;
                        	}
			}
			$c = 0; $motif = ""; $nseq++;
		} else {
			my @w = split(//);
			for my $i (0..$#w) {
				my $b = $w[$i];
				if (!($b =~ /N/)) {$gensize++;}
				if (!(defined($bp{$b}))) {
					$bp{$b} = 1;
				} else {
					$bp{$b}++;
				}
				$c++;
				if ($c <= $k) {
					$motif .= $b;
				} elsif ($c > $k) {
					if (!(defined($motifs{$motif}))) {
						$motifs{$motif} = 1;
					} else {
						$motifs{$motif}++;
					}
					$motif = substr($motif,1).$b;
				}
			}
		}
	}
	close(G);

        if (!(defined($motifs{$motif}))) {
	        $motifs{$motif} = 1;
        } else {
                $motifs{$motif}++;
        }

	my $quant = ($nseq)*($k - 1);
	my $gensize2 = $gensize - $quant;

	my %pb = ();
	$pb{'A'} = $bp{'A'}/$gensize;
        $pb{'C'} = $bp{'C'}/$gensize;
        $pb{'G'} = $bp{'G'}/$gensize;
        $pb{'T'} = $bp{'T'}/$gensize;

	open (OF, ">$ofile") or die "Cannot open outfile $ofile $!\n";
	print OF "Motif\tObs\tExp\tScore\n";
	for my $m (sort keys %{motifs}) {
		if (!($m =~ /N/)) {
			my $score = 0;
			my $nn = $gensize2;
			my $mm = $m;
			my @ww = split(//,$m);
			for my $j (0..$#ww) {
				my $bb = $ww[$j];
				$nn *= $pb{$bb};
			}
			my $nn2 = sprintf("%.3f",$nn);
			$score = $motifs{$mm}*log($motifs{$mm}/$nn);
			my $score2 = sprintf("%.3f",$score);
			print OF "$mm\t$motifs{$mm}\t$nn2\t$score2\n";
		}
	}
	close(OF);
}
